%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function fittingOutput = checkFittingonRealPatchAllScanlinesRef(patch_scandata_all, ...
                                                                   patch_scanline_all, ...
                                                                   gamma_model)
                                                               
% Get the CRF model and num coeffs
model_type = gamma_model.model_type;
num_coeffs = gamma_model.num_coeffs;
if(~strcmp(model_type, 'GGCM') && ~strcmp(model_type, 'Polynomial'))
    error('Currently only GGCM or Polynomial model is supported!');
end
if(num_coeffs<1)
    error('Number of coefficients to be atleast 1!');
end
true_cam_I      = gamma_model.true_cam_I;
if(isempty(true_cam_I))
    error('True value of I cannot be empty!');
end
[dims, patchSize]  = size(patch_scandata_all{1});
if(dims ~= 3)
    error('Expected dims of patch in 3xpatchSize!');
end

% Setup contraints on the coeffs (if any)
if(strcmp(model_type, 'GGCM'))
    if(num_coeffs == 1)
        coeffs_lb = 0.1;
        coeffs_ub = 1.0;
    elseif(num_coeffs == 2)
        coeffs_lb = [0.1, -1.0];
        coeffs_ub = [1.0,  1.0];
    else
        coeffs_lb = [];
        coeffs_ub = [];
    end
    gamma_coeffs_init = [1.0, zeros(num_coeffs-1, 1)];
else
    coeffs_lb = [];
    coeffs_ub = [];
    gamma_coeffs_init = [0.0, zeros(num_coeffs-1, 1)];
end

% For all the scalines in the patch
gamma_coeffs_pred_all = cell(patchSize, 1);
mimima_dists_all      = zeros(patchSize ,1);
for n_line=1:patchSize
    % Get patch_scandata 
    patch_scandata = patch_scandata_all{n_line};
    patch_scanline = patch_scanline_all{n_line};
    % Check fitting on the patch_scandata 
    datatocheck = patch_scandata;
    dispOptions = {'final', 'iter', 'iter-detailed', 'off'};
    optimOptions= optimset('Display', dispOptions{4}, 'MaxFunEvals', 1e9, ...
                           'MaxIter', 1e4, 'LargeScale', 'off');
    %FittingDistanceFunction(gamma_coeffs_init)
    gamma_coeffs_pred_all{n_line} = fmincon(@FittingDistanceFunction, gamma_coeffs_init, ...
                                            [], [], [], [], coeffs_lb, coeffs_ub, [], optimOptions);
    mimima_dists_all(n_line)      = FittingDistanceFunction(gamma_coeffs_pred_all{n_line});
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function costvalue = FittingDistanceFunction(coeffs)
    % disp(coeffs);
    % Transform the points
    dummydata = linspace(0, 1, 1000);
    if(strcmp(model_type, 'GGCM'))
        datatochecktrn = crf_ggcm(datatocheck, coeffs);
        dummydatatrn   = crf_ggcm(dummydata, coeffs);
    elseif(strcmp(model_type, 'Polynomial'))
        datatochecktrn = crf_poly(datatocheck, coeffs);
        dummydatatrn   = crf_poly(dummydata, coeffs);
    end
    datatochecktrn_meannormed = meanNormalizeData(datatochecktrn);
    if(any(dummydatatrn(:)<dummydata(:).^(1/0.1))) % Avoid trivial solns!
        costvalue = 1e64;
    elseif(any(dummydatatrn(:)>dummydata(:))) % Monotonicity is must!
        costvalue = 1e64;
    else
        % 3D line fitting on the linearized/transformed points
        [~, dists2] = fitline3d(datatochecktrn_meannormed);
        costvalue1  = mean(dists2);
        costvalue = costvalue1;
    end
    if(isnan(costvalue))
        %warning('NaN detected in costvalue!');
        %costvalue = 1e64;
        error('NaN detected in costvalue!');
    end
end
fittingOutput.gamma_coeffs_pred_all = gamma_coeffs_pred_all;
end